package com.loafle.overflow.central.module.infra.service;

import com.loafle.overflow.central.module.infra.dao.InfraHostPortDAO;
import com.loafle.overflow.central.module.meta.service.CentralMetaPortTypeService;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.infra.InfraHostPort;
import com.loafle.overflow.model.meta.MetaPortType;
import com.loafle.overflow.service.central.infra.InfraHostPortService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * Created by insanity on 17. 6. 28.
 */

@Service("InfraHostPortService")
public class CentralInfraHostPortService implements InfraHostPortService {
  @Autowired
  InfraHostPortDAO infraHostPortDAO;

  @Autowired
  CentralMetaPortTypeService metaPortTypeService;

  public InfraHostPort validate(InfraHostPort infraHostPort) throws OverflowException {
    if (null == infraHostPort) {
      throw new OverflowException("InfraHostPort is not valid");
    }

    MetaPortType metaPortType = infraHostPort.getMetaPortType();

    if (null == metaPortType) {
      throw new OverflowException("MetaPortType is not valid");
    }

    if (null == metaPortType.getId()) {
      String metaPortTypeKey = metaPortType.getKey();
      if (null == metaPortTypeKey) {
        throw new OverflowException("Key of MetaPortType is not valid");
      }

      metaPortType = this.metaPortTypeService.readByKey(metaPortTypeKey);
      if (null == metaPortType) {
        throw new OverflowException(String.format("Key[%s] of MetaPortType is not valid", metaPortTypeKey));
      }
      infraHostPort.setMetaPortType(metaPortType);
    }

    if (null == infraHostPort.getPort()) {
      throw new OverflowException("Number of Port is not valid");
    }

    return infraHostPort;
  }

  public InfraHostPort regist(InfraHostPort infraHostPort) throws OverflowException {
    infraHostPort = this.validate(infraHostPort);

    if (null == infraHostPort.getInfraHost()) {
      throw new OverflowException("InfraHost is not valid");
    }

    return this.infraHostPortDAO.save(infraHostPort);
  }

  public InfraHostPort read(Long id) throws OverflowException {
    return this.infraHostPortDAO.findById(id).get();
  }

  public InfraHostPort readByInfraHostIDAndMetaPortTypeKeyAndPort(Long infraHostID, String metaPortTypeKey,
      Integer port) throws OverflowException {
    return this.infraHostPortDAO.findByInfraHostIdAndMetaPortTypeKeyAndPort(infraHostID, metaPortTypeKey, port);
  }

  public InfraHostPort readByInfraHostInfraZoneIDAndMetaPortTypeKeyAndPort(Long infraHostInfraZoneID,
      String metaPortTypeKey, Integer port) throws OverflowException {
    return this.infraHostPortDAO.findByInfraHostInfraZoneIdAndMetaPortTypeKeyAndPort(infraHostInfraZoneID, metaPortTypeKey, port);
  }
}
